from allure import *
from requests_toolbelt import \
    MultipartEncoder as FormEncoder
from kuto.api.case import Case
from kuto.page import Page
from kuto.running.runner import main
from kuto.utils.config import config
from kuto.utils.log import logger
from kuto.utils.pytest_util import \
    depend, order, data


__version__ = "0.0.64"
__description__ = "全平台自动化测试框架"



import time

from kuto.api.request import HttpReq
from kuto.utils.log import logger


class Case(HttpReq):
    """
    测试用例基类，所有测试用例需要继承该类
    """

    # ---------------------初始化-------------------------------
    def start_class(self):
        """
        Hook method for setup_class fixture
        :return:
        """
        pass

    def end_class(self):
        """
        Hook method for teardown_class fixture
        :return:
        """
        pass

    @classmethod
    def setup_class(cls):
        cls().start_class()

    @classmethod
    def teardown_class(cls):
        cls().end_class()

    def start(self):
        """
        Hook method for setup_method fixture
        :return:
        """
        pass

    def end(self):
        """
        Hook method for teardown_method fixture
        :return:
        """
        pass

    def setup_method(self):
        self.start_time = time.time()
        self.start()

    def teardown_method(self):
        self.end()
        take_time = time.time() - self.start_time
        logger.info("用例耗时: {:.2f} s".format(take_time))

    @staticmethod
    def sleep(n: float):
        """休眠"""
        logger.info(f"暂停: {n}s")
        time.sleep(n)







import time

from typing import Union

from kuto.api.request import HttpReq
from kuto.win.driver import Driver

from kuto.utils.config import config
from kuto.utils.log import logger


class Case(HttpReq):
    """
    测试用例基类，所有测试用例需要继承该类
    """

    driver: Union[Driver] = None

    # ---------------------初始化-------------------------------
    def start_class(self):
        """
        Hook method for setup_class fixture
        :return:
        """
        pass

    def end_class(self):
        """
        Hook method for teardown_class fixture
        :return:
        """
        pass

    @classmethod
    def setup_class(cls):
        cls().start_class()

    @classmethod
    def teardown_class(cls):
        cls().end_class()

    def start(self):
        """
        Hook method for setup_method fixture
        :return:
        """
        pass

    def end(self):
        """
        Hook method for teardown_method fixture
        :return:
        """
        pass

    def setup_method(self):
        self.start_time = time.time()

        pkg_name = config.get_app("pkg_name")
        self.driver = Driver(pkg_name=pkg_name)

        self.start()

    def teardown_method(self):
        self.end()

        if config.get_app("auto_start") is True:
            self.driver.stop_app()

        take_time = time.time() - self.start_time
        logger.info("用例耗时: {:.2f} s".format(take_time))

    @staticmethod
    def sleep(n: float):
        """休眠"""
        logger.info(f"暂停: {n}s")
        time.sleep(n)

    def screenshot(self, name: str):
        """截图"""
        self.driver.screenshot(name)






"""
@Author: kang.yang
@Date: 2023/9/14 14:43
"""
from kuto.mac.driver import Driver
from kuto.mac.element import CoorElem, ImageElem
from kuto.mac.case import Case


__all__ = ["Case", "Driver", "CoorElem", "ImageElem"]



import json
import time

from typing import Union
from urllib import parse

from kuto.web.driver import Driver
from kuto.web.element import Elem
from kuto.api.request import HttpReq

from kuto.utils.config import config
from kuto.utils.log import logger
from kuto.utils.exceptions import KError


class Case(HttpReq):
    """
    测试用例基类，所有测试用例需要继承该类
    """

    driver: Union[Driver] = None

    # ---------------------初始化-------------------------------
    def start_class(self):
        """
        Hook method for setup_class fixture
        :return:
        """
        pass

    def end_class(self):
        """
        Hook method for teardown_class fixture
        :return:
        """
        pass

    @classmethod
    def setup_class(cls):
        cls().start_class()

    @classmethod
    def teardown_class(cls):
        cls().end_class()

    def start(self):
        """
        Hook method for setup_method fixture
        :return:
        """
        pass

    def end(self):
        """
        Hook method for teardown_method fixture
        :return:
        """
        pass

    def setup_method(self):
        self.start_time = time.time()

        browserName = config.get_web("browser_name")
        headless = config.get_web("headless")
        state = config.get_web("state")
        if state:
            state_json = json.loads(state)
            self.driver = Driver(browserName=browserName, headless=headless, state=state_json)
        else:
            self.driver = Driver(browserName=browserName, headless=headless)

        self.start()

    def teardown_method(self):
        self.end()

        self.driver.close()

        take_time = time.time() - self.start_time
        logger.info("用例耗时: {:.2f} s".format(take_time))

    @staticmethod
    def sleep(n: float):
        """休眠"""
        logger.info(f"暂停: {n}s")
        time.sleep(n)

    def open_url(self, url=None):
        """浏览器打开页面"""
        # 拼接域名
        if url is None:
            base_url = config.get_web("base_url")
            if not base_url:
                raise KError('base_url is null')
            url = base_url
        else:
            if "http" not in url:
                base_url = config.get_web("base_url")
                if not base_url:
                    raise KError('base_url is null')
                url = parse.urljoin(base_url, url)
        # 访问页面
        self.driver.open_url(url)
        # 设置cookies
        cookies = config.get_web("cookies")
        if cookies:
            self.driver.set_cookies(cookies)

    def switch_tab(self, **kwargs):
        """切换到新页签，需要先定位导致跳转的元素"""
        locator = Elem(self.driver, **kwargs)
        self.driver.switch_tab(locator)

    def screenshot(self, name: str):
        """截图"""
        self.driver.screenshot(name)

    def assert_title(self, title: str, timeout=5):
        """断言页面title，web端使用"""
        self.driver.assert_title(title, timeout=timeout)

    def assert_url(self, url: str = None, timeout=5):
        """断言页面url，web端使用"""
        # 拼接域名
        if url is None:
            base_url = config.get_web("base_url")
            if not base_url:
                raise KError('base_url is null')
            url = base_url + "/"
        else:
            if "http" not in url:
                base_url = config.get_web("base_url")
                if not base_url:
                    raise KError('base_url is null')
                url = parse.urljoin(base_url, url)
        self.driver.assert_url(url, timeout=timeout)






import time
from typing import Union

from filelock import FileLock

from kuto.android.driver import Driver
from kuto.api.request import HttpReq

from kuto.utils.config import config, free_config
from kuto.utils.log import logger
from kuto.utils.exceptions import KError


class Case(HttpReq):
    """
    测试用例基类，所有测试用例需要继承该类
    """

    driver: Union[Driver] = None

    # ---------------------初始化-------------------------------
    def start_class(self):
        """
        Hook method for setup_class fixture
        :return:
        """
        pass

    def end_class(self):
        """
        Hook method for teardown_class fixture
        :return:
        """
        pass

    @classmethod
    def setup_class(cls):
        cls().start_class()

    @classmethod
    def teardown_class(cls):
        cls().end_class()

    def start(self):
        """
        Hook method for setup_method fixture
        :return:
        """
        pass

    def end(self):
        """
        Hook method for teardown_method fixture
        :return:
        """
        pass

    def setup_method(self):
        self.start_time = time.time()
        # 加一段逻辑支持多进程和设备调度
        logger.info("开始获取空闲设备")
        timeout = 30  # 后面再考虑要不要在runner中加一个超时设置
        while timeout > 0:
            with FileLock("session.lock"):
                device_id = free_config.get_random_device()
            if device_id:
                logger.info(f"获取空闲设备成功: {device_id}")
                logger.info(f"剩余空闲设备列表: {free_config.get_all_device()}")
                break
            logger.info("未找到空闲设备，休息3秒")
            timeout -= 3
            time.sleep(3)
        else:
            logger.info(f"获取空闲设备失败!!!")
            logger.info(f"剩余空闲设备列表: {free_config.get_all_device()}")
            raise KError("获取空闲设备失败!!!")

        pkg_name = config.get_app("pkg_name")
        self.driver = Driver(device_id=device_id, pkg_name=pkg_name)

        if config.get_app("auto_start") is True:
            self.driver.start_app()
        self.start()

    def teardown_method(self):
        self.end()
        if config.get_app("auto_start") is True:
            self.driver.stop_app()

        # 加一段逻辑支持多进程和设备调度
        device_id = self.driver.device_id
        logger.info(f"用例结束释放设备: {device_id}")
        with FileLock("session.lock"):
            devices = free_config.get('devices')
            if device_id not in devices:
                free_config.add_devices([self.driver.device_id])
        logger.info(f"剩余空闲设备列表: {free_config.get_all_device()}")

        take_time = time.time() - self.start_time
        logger.info("用例耗时: {:.2f} s".format(take_time))

    @staticmethod
    def sleep(n: float):
        """休眠"""
        logger.info(f"暂停: {n}s")
        time.sleep(n)

    def screenshot(self, name: str):
        """截图"""
        self.driver.screenshot(name)

    # 断言
    def assert_act(self, activity_name: str, timeout=5):
        """断言当前activity，安卓端使用"""
        self.driver.assert_act(activity_name, timeout=timeout)






"""
@Author: kang.yang
@Date: 2023/8/21 17:05
"""
import kuto
from kuto.ios import Elem
from kuto.ocr import OCRElem

"""
ocr识别可以配合安卓应用或者IOS应用进行使用
"""


class OcrPage(kuto.Page):
    searchBtn = Elem(text="搜索", className="XCUIElementTypeSearchField", desc='搜索框入口')
    searchInput = Elem(className="XCUIElementTypeSearchField", desc='搜索框')
    searchResult = Elem(xpath="//Table/Cell[2]", desc='搜索结果')
    schoolEntry = OCRElem(text="校园场馆", pos=12)




import kuto
from kuto import FormEncoder


class TestApiDemo(kuto.Case):
    """接口demo"""

    @kuto.title("一般请求")
    def test_normal_req(self):
        payload = {"type": 2}
        headers = {
            "user-agent-web": "X/b67aaff2200d4fc2a2e5a079abe78cc6"
        }
        self.post('/qzd-bff-app/qzd/v1/home/getToolCardListForPc',
                  json=payload, headers=headers)
        self.assert_eq('code', 0)

    # @kuto.title("文件上传")
    # def test_upload_file(self):
    #     path = '/qzd-bff-patent/patent/batch/statistics/upload'
    #     files = {'static': open('../static/号码上传模板_1.xlsx', 'rb')}
    #     self.post(path, files=files)
    #     self.assert_eq('code', 0)
    #
    # @kuto.title("form请求")
    # def test_form_req(self):
    #     url = '/qzd-bff-patent/image-search/images'
    #     file_data = (
    #         'logo.png',
    #         open('../static/logo.png', 'rb'),
    #         'image/png'
    #     )
    #     fields = {
    #         'key1': 'value1',  # 参数
    #         'imageFile': file_data  # 文件
    #     }
    #     form_data = FormEncoder(fields=fields)
    #     headers = {'Content-Type': form_data.content_type}
    #     self.post(url, data=form_data, headers=headers)
    #     self.assert_eq("code", 0)


if __name__ == '__main__':
    """仅执行本模块"""
    kuto.main(
        platform="api",
        host='https://app-test.qizhidao.com'
    )



"""
@Author: kang.yang
@Date: 2023/9/15 10:19
"""
import kuto
from kuto.mac import Case

from page.mac_page import MacPage


class TestMacDemo(Case):
    """Mac应用demo"""

    def start(self):
        self.mac_page = MacPage(self.driver)

    def test_1(self):
        self.mac_page.num_3.click()
        self.mac_page.x.click()
        self.mac_page.num_5.click()
        self.mac_page.equal.click()
        self.driver.screenshot("计算结果")


if __name__ == '__main__':
    """仅执行本模块"""
    kuto.main(
        platform='mac',
        pkg_name='Calculator',
    )



"""
pip install -i https://pypi.tuna.tsinghua.edu.cn/simple kuto[reader]
@Author: kang.yang
@Date: 2023/9/20 13:51
"""
import json
import yaml

from kuto.utils.excel import Excel, CSV
from kuto.utils.pytest_util import data


# 从json文件获取数据进行参数化
def file_data(file=None, key=None, row=None):
    # logger.debug(config.get_env())

    """
    @param row: 第几行，针对csv和excel文件
    @param file: 文件名
    @param key: 针对json和yaml文件
    """
    file_path = file  # 去掉文件查找机制，不好理解，用处也不大

    # logger.debug(file_path)
    if file_path.endswith(".json"):
        content = read_json(file_path, key)
    elif file_path.endswith(".yml"):
        content = read_yaml(file_path, key)
    elif file_path.endswith(".csv"):
        content = read_csv(file_path, row)
    elif file_path.endswith(".xlsx"):
        content = read_excel(file_path, row)
    else:
        raise TypeError("不支持的文件类型，仅支持json、yml、csv、xlsx")

    if content:
        return data(content)
    else:
        raise ValueError('数据不能为空')


def read_json(file_path, key=None):
    """
    读取json文件中的指定key
    @return
    """
    with open(file_path, 'r', encoding='utf-8') as f:
        json_data = json.load(f)
    if isinstance(json_data, list):
        return json_data
    else:
        if key:
            return json_data[key]
        raise ValueError('key不能为空')


def read_yaml(file_path, key=None):
    """
    读取yaml文件中的指定key
    @param file_path:
    @param key:
    """
    with open(file_path, 'r', encoding='utf-8') as f:
        yaml_data = yaml.load(f, Loader=yaml.FullLoader)
    if isinstance(yaml_data, list):
        return yaml_data
    else:
        if key:
            return yaml_data[key]
        raise ValueError('key不能为空')


def read_csv(file_path, row=None):
    """
    读取csv文件中的指定行
    @param file_path: 文件名
    @param row: 行数，从1开始
    """
    csv_file = CSV(file_path)
    if row:
        csv_data = [csv_file.read_row_index(row)]
    else:
        csv_data = csv_file.read_all()
    return csv_data


def read_excel(file_path, row=None):
    """
    读取excel文件中的指定行，暂时只支持读取第一个sheet
    @param file_path: 文件名
    @param row: 行数，从1开始
    """
    excel_file = Excel(file_path)
    if row:
        excel_data = [excel_file.read_row_index(row)]
    else:
        excel_data = excel_file.read_all()
    return excel_data
